@@ -12,6 +12,7 @@ from typing import Any, Callable, Dict, Generator, List, Iterable, Tuple, Union
 
 from . import metrics
 from . import parser
+from . import pings
 from . import util
 
 
@@ -34,6 +35,20 @@ def _split_words(name: str) -> List[str]:
     return re.split("[._]", name)
 
 
+def _english_list(items: List[str]) -> str:
+    """
+    Helper function to format a list [A, B, C] as "'A', 'B', or 'C'".
+    """
+    if len(items) == 0:
+        return ""
+    elif len(items) == 1:
+        return f"'{items[0]}'"
+    else:
+        return "{}, or '{}'".format(
+            ", ".join([f"'{x}'" for x in items[:-1]]), items[-1]
+        )
+
+
 def _hamming_distance(str1: str, str2: str) -> int:
     """
     Count the # of differences between strings str1 and str2,
@@ -73,9 +88,11 @@ def check_common_prefix(
     if i > 0:
         common_prefix = "_".join(first[:i])
         yield (
-            "Within category '{}', all metrics begin with prefix "
-            "'{}'. Remove prefixes and (possibly) rename category."
-        ).format(category_name, common_prefix)
+            f"Within category '{category_name}', all metrics begin with "
+            f"prefix '{common_prefix}'."
+            "Remove the prefixes on the metric names and (possibly) "
+            "rename the category."
+        )
 
 
 def check_unit_in_name(
@@ -114,16 +131,18 @@ def check_unit_in_name(
             or unit_in_name == time_unit.name
         ):
             yield (
-                "Suffix '{}' is redundant with time_unit. " "Only include time_unit."
-            ).format(unit_in_name)
+                f"Suffix '{unit_in_name}' is redundant with time_unit "
+                f"'{time_unit.name}'. Only include time_unit."
+            )
         elif (
             unit_in_name in TIME_UNIT_ABBREV.keys()
             or unit_in_name in TIME_UNIT_ABBREV.values()
         ):
             yield (
-                "Suffix '{}' doesn't match time_unit. "
+                f"Suffix '{unit_in_name}' doesn't match time_unit "
+                f"'{time_unit.name}'. "
                 "Confirm the unit is correct and only include time_unit."
-            ).format(unit_in_name)
+            )
 
     elif memory_unit is not None:
         if (
@@ -131,23 +150,27 @@ def check_unit_in_name(
             or unit_in_name == memory_unit.name
         ):
             yield (
-                "Suffix '{}' is redundant with memory_unit. "
+                f"Suffix '{unit_in_name}' is redundant with memory_unit "
+                f"'{memory_unit.name}'. "
                 "Only include memory_unit."
-            ).format(unit_in_name)
+            )
         elif (
             unit_in_name in MEMORY_UNIT_ABBREV.keys()
             or unit_in_name in MEMORY_UNIT_ABBREV.values()
         ):
             yield (
-                "Suffix '{}' doesn't match memory_unit. "
+                f"Suffix '{unit_in_name}' doesn't match memory_unit "
+                f"{memory_unit.name}'. "
                 "Confirm the unit is correct and only include memory_unit."
-            ).format(unit_in_name)
+            )
 
     elif unit is not None:
         if unit_in_name == unit:
             yield (
-                "Suffix '{}' is redundant with unit param. " "Only include unit."
-            ).format(unit_in_name)
+                f"Suffix '{unit_in_name}' is redundant with unit param "
+                f"'{unit}'. "
+                "Only include unit."
+            )
 
 
 def check_category_generic(
@@ -159,7 +182,10 @@ def check_category_generic(
     GENERIC_CATEGORIES = ["metrics", "events"]
 
     if category_name in GENERIC_CATEGORIES:
-        yield "Category '{}' is too generic.".format(category_name)
+        yield (
+            f"Category '{category_name}' is too generic. "
+            f"Don't use {_english_list(GENERIC_CATEGORIES)} for category names"
+        )
 
 
 def check_bug_number(
@@ -169,9 +195,10 @@ def check_bug_number(
 
     if len(number_bugs):
         yield (
-            "For bugs {}: "
-            "Bug numbers are deprecated and should be changed to full URLs."
-        ).format(", ".join(number_bugs))
+            f"For bugs {', '.join(number_bugs)}: "
+            "Bug numbers are deprecated and should be changed to full URLs. "
+            "For example, use 'http://bugzilla.mozilla.org/12345' instead of '12345'."
+        )
 
 
 def check_valid_in_baseline(
@@ -182,22 +209,18 @@ def check_valid_in_baseline(
     if not allow_reserved and "baseline" in metric.send_in_pings:
         yield (
             "The baseline ping is Glean-internal. "
-            "User metrics should go into the 'metrics' ping or custom pings."
+            "Remove 'baseline' from the send_in_pings array."
         )
 
 
 def check_misspelled_pings(
     metric: metrics.Metric, parser_config: Dict[str, Any] = {}
 ) -> LintGenerator:
-    builtin_pings = ["metrics", "events"]
-
     for ping in metric.send_in_pings:
-        for builtin in builtin_pings:
+        for builtin in pings.RESERVED_PING_NAMES:
             distance = _hamming_distance(ping, builtin)
             if distance == 1:
-                yield ("Ping '{}' seems misspelled. Did you mean '{}'?").format(
-                    ping, builtin
-                )
+                yield f"Ping '{ping}' seems misspelled. Did you mean '{builtin}'?"
 
 
 def check_user_lifetime_expiration(
@@ -206,28 +229,33 @@ def check_user_lifetime_expiration(
 
     if metric.lifetime == metrics.Lifetime.user and metric.expires != "never":
         yield (
-            "Metrics with `user` lifetime cannot have an expiration date. "
-            "They live as long as the user profile does."
+            "Metrics with 'user' lifetime cannot have an expiration date. "
+            "They live as long as the user profile does. "
+            "Set expires to 'never'."
         )
 
 
 # The checks that operate on an entire category of metrics:
 #    {NAME: (function, is_error)}
-CATEGORY_CHECKS = {
+CATEGORY_CHECKS: Dict[
+    str, Tuple[Callable[[str, Iterable[metrics.Metric]], LintGenerator], CheckType]
+] = {
     "COMMON_PREFIX": (check_common_prefix, CheckType.error),
     "CATEGORY_GENERIC": (check_category_generic, CheckType.error),
-}  # noqa type: Dict[str, Tuple[Callable[[str, Iterable[metrics.Metric]], LintGenerator], CheckType]]
+}
 
 
 # The checks that operate on individual metrics:
 #     {NAME: (function, is_error)}
-INDIVIDUAL_CHECKS = {
+INDIVIDUAL_CHECKS: Dict[
+    str, Tuple[Callable[[metrics.Metric, dict], LintGenerator], CheckType]
+] = {
     "UNIT_IN_NAME": (check_unit_in_name, CheckType.error),
     "BUG_NUMBER": (check_bug_number, CheckType.error),
     "BASELINE_PING": (check_valid_in_baseline, CheckType.error),
     "MISSPELLED_PING": (check_misspelled_pings, CheckType.error),
     "USER_LIFETIME_EXPIRATION": (check_user_lifetime_expiration, CheckType.warning),
-}  # type: Dict[str, Tuple[Callable[[metrics.Metric, dict], LintGenerator], CheckType]]
+}
 
 
 class GlinterNit:
@@ -238,8 +266,9 @@ class GlinterNit:
         self.check_type = check_type
 
     def format(self):
-        return "{}: {}: {}: {}".format(
-            self.check_type.name.upper(), self.check_name, self.name, self.msg
+        return (
+            f"{self.check_type.name.upper()}: {self.check_name}: "
+            f"{self.name}: {self.msg}"
         )
 
 
@@ -253,7 +282,7 @@ def lint_metrics(
     :param file: The stream to write errors to.
     :returns: List of nits.
     """
-    nits = []  # type: List[GlinterNit]
+    nits: List[GlinterNit] = []
     for (category_name, category) in sorted(list(objs.items())):
         if category_name == "pings":
             continue
@@ -299,9 +328,9 @@ def lint_metrics(
                                 "SUPERFLUOUS_NO_LINT",
                                 ".".join([metric.category, metric.name]),
                                 (
-                                    "Superfluous no_lint entry '{}'. "
+                                    f"Superfluous no_lint entry '{check_name}'. "
                                     "Please remove it."
-                                ).format(check_name),
+                                ),
                                 CheckType.warning,
                             )
                         )
@@ -334,7 +363,7 @@ def lint_yaml_files(input_filepaths: Iterable[Path], file=sys.stderr) -> List:
 
     # Generic type since the actual type comes from yamllint, which we don't
     # control.
-    nits = []  # type: List
+    nits: List = []
     for path in input_filepaths:
         # yamllint needs both the file content and the path.
         file_content = None
@@ -347,7 +376,7 @@ def lint_yaml_files(input_filepaths: Iterable[Path], file=sys.stderr) -> List:
     if len(nits):
         print("Sorry, Glean found some glinter nits:", file=file)
         for (path, p) in nits:
-            print("{} ({}:{}) - {}".format(path, p.line, p.column, p.message))
+            print(f"{path} ({p.line}:{p.column}) - {p.message}")
         print("", file=file)
         print("Please fix the above nits to continue.", file=file)
 
